//
// Copyright (c) 2013 Samsung Electronics Co., Ltd.
//
// Licensed under the Flora License, Version 1.1 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://floralicense.org/license/
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an AS IS BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <new>
#include <FUi.h>

#include "SchedulerFormFactory.h"
#include "EventListForm.h"
#include "CreateEventForm.h"
#include "EventDetailForm.h"
#include "EditEventForm.h"
#include "SetRecurrenceForm.h"

using namespace Tizen::Ui::Scenes;

// Definitions of extern.
const wchar_t* FORM_LIST = L"EventListForm";
const wchar_t* FORM_DETAIL = L"EventDetailForm";
const wchar_t* FORM_CREATION = L"CreateEventForm";
const wchar_t* FORM_EDITION = L"EditEventForm";
const wchar_t* FORM_RECURRENCE = L"SetRecurrenceForm";

SchedulerFormFactory::SchedulerFormFactory(void)
{
}

SchedulerFormFactory::~SchedulerFormFactory(void)
{
}

Tizen::Ui::Controls::Form*
SchedulerFormFactory::CreateFormN(const Tizen::Base::String& formId, const Tizen::Ui::Scenes::SceneId& sceneId)
{
	SceneManager* pSceneManager = SceneManager::GetInstance();
	AppAssert(pSceneManager);
	Tizen::Ui::Controls::Form* pNewForm = null;

	if (formId == FORM_LIST)
	{
		EventListForm* pForm = new (std::nothrow) EventListForm();
		pForm->Initialize();
		pSceneManager->AddSceneEventListener(sceneId, *pForm);
		pNewForm = pForm;
	}
	else if (formId == FORM_CREATION)
	{
		CreateEventForm* pForm = new (std::nothrow) CreateEventForm();
		pForm->Initialize();
		pSceneManager->AddSceneEventListener(sceneId, *pForm);
		pNewForm = pForm;
	}
	else if (formId == FORM_DETAIL)
	{
		EventDetailForm* pForm = new (std::nothrow) EventDetailForm();
		pForm->Initialize();
		pSceneManager->AddSceneEventListener(sceneId, *pForm);
		pNewForm = pForm;
	}
	else if (formId == FORM_EDITION)
	{
		EditEventForm* pForm = new (std::nothrow) EditEventForm();
		pForm->Initialize();
		pSceneManager->AddSceneEventListener(sceneId, *pForm);
		pNewForm = pForm;
	}
	else if (formId == FORM_RECURRENCE)
	{
		SetRecurrenceForm* pForm = new (std::nothrow) SetRecurrenceForm();
		pForm->Initialize();
		pSceneManager->AddSceneEventListener(sceneId, *pForm);
		pNewForm = pForm;
	}

	return pNewForm;
}
